@@ -18,24 +18,12 @@ import os
 
 import gtk.gdk
 
-from bzrlib.config import config_dir
+from bzrlib import config
 import bzrlib.util.configobj.configobj as configobj
 
 
-gannotate_configspec = (
-    "[window]",
-    "width = integer(default=750)",
-    "height = integer(default=550)",
-    "maximized = boolean(default=False)",
-    "x = integer(default=0)",
-    "y = integer(default=0)",
-    "pane_position = integer(default=325)",
-    "[spans]",
-    "max_custom_spans = integer(default=4)",
-    "custom_spans = float_list()"
-)
-
-gannotate_config_filename = os.path.join(config_dir(), "gannotate.conf")
+def gannotate_config_filename():
+    return os.path.join(config.config_dir(), "gannotate.conf")
 
 
 class GAnnotateConfig(configobj.ConfigObj):
@@ -47,20 +35,20 @@ class GAnnotateConfig(configobj.ConfigObj):
     """
 
     def __init__(self, window):
-        configobj.ConfigObj.__init__(self, gannotate_config_filename,
-                                     configspec=gannotate_configspec)
+        configobj.ConfigObj.__init__(self, gannotate_config_filename())
         self.window = window
         self.pane = window.pane
-        
-        self.initial_comment = ["gannotate plugin configuration"]
-        self['window']['width'] = 750
-        self['window']['height'] = 550
-        self['window']['maximized'] = False
-        self['window']['x'] = 0
-        self['window']['y'] = 0
-        self['window']['pane_position'] = 325
-        self['spans']['max_custom_spans'] = 4
-        self['spans']['custom_spans'] = []
+
+        if 'window' not in self:
+            # Set default values, configobj expects strings here
+            self.initial_comment = ["gannotate plugin configuration"]
+            self['window'] = {}
+            self['window']['width'] = '750'
+            self['window']['height'] = '550'
+            self['window']['maximized'] = 'False'
+            self['window']['x'] = '0'
+            self['window']['y'] = '0'
+            self['window']['pane_position'] = '325'
 
         self.apply()
         self._connect_signals()
@@ -68,16 +56,14 @@ class GAnnotateConfig(configobj.ConfigObj):
     def apply(self):
         """Apply properties and such from gannotate.conf, or
         gannotate_configspec defaults."""
-        self.pane.set_position(self["window"]["pane_position"])
-        self.window.set_default_size(self["window"]["width"],
-                                     self["window"]["height"])
-        self.window.move(self["window"]["x"], self["window"]["y"])
+        self.pane.set_position(self['window'].as_int('pane_position'))
+        self.window.set_default_size(self['window'].as_int('width'),
+                                     self['window'].as_int('height'))
+        self.window.move(self['window'].as_int('x'), self['window'].as_int('y'))
 
-        if self["window"]["maximized"]:
+        if self['window'].as_bool('maximized'):
             self.window.maximize()
 
-        # XXX Don't know how to set an empty list as default in
-        # gannotate_configspec.
     def _connect_signals(self):
         self.window.connect("destroy", self._write)
         self.window.connect("configure-event", self._save_window_props)
@@ -91,9 +77,7 @@ class GAnnotateConfig(configobj.ConfigObj):
             self["window"]["width"], self["window"]["height"] = w.get_size()
             self["window"]["x"], self["window"]["y"] = w.get_position()
             maximized = False
-
         self["window"]["maximized"] = maximized
-        
         return False
 
     def _save_pane_props(self, w, gparam):
@@ -102,11 +86,6 @@ class GAnnotateConfig(configobj.ConfigObj):
 
         return False
 
-    def _save_custom_spans(self, w, *args):
-        self["spans"]["custom_spans"] = w.custom_spans
-
-        return False
-
     def _write(self, *args):
         self.write()
 
