@@ -1,11 +1,10 @@
-# -*- coding: UTF-8 -*-
 """Difference window.
 
 This module contains the code to manage the diff window which shows
 the changes made between two revisions on a branch.
 """
 
-__copyright__ = "Copyright © 2005 Canonical Ltd."
+__copyright__ = "Copyright 2005 Canonical Ltd."
 __author__    = "Scott James Remnant <scott@ubuntu.com>"
 
 
@@ -18,9 +17,10 @@ import pango
 import os
 import re
 import sys
+from xml.etree.ElementTree import Element, SubElement, tostring
 
 try:
-    import gtksourceview
+    import gtksourceview2
     have_gtksourceview = True
 except ImportError:
     have_gtksourceview = False
@@ -46,6 +46,14 @@ from bzrlib.plugins.gtk.window import Window
 from dialog import error_dialog, info_dialog, warning_dialog
 
 
+def fallback_guess_language(slm, content_type):
+    for lang_id in slm.get_language_ids():
+        lang = slm.get_language(lang_id)
+        if "text/x-patch" in lang.get_mime_types():
+            return lang
+    return None
+
+
 class SelectCancelled(Exception):
 
     pass
@@ -64,16 +72,18 @@ class DiffFileView(gtk.ScrolledWindow):
         self.set_shadow_type(gtk.SHADOW_IN)
 
         if have_gtksourceview:
-            self.buffer = gtksourceview.SourceBuffer()
-            slm = gtksourceview.SourceLanguagesManager()
-            gsl = slm.get_language_from_mime_type("text/x-patch")
+            self.buffer = gtksourceview2.Buffer()
+            slm = gtksourceview2.LanguageManager()
+            guess_language = getattr(gtksourceview2.LanguageManager, 
+                "guess_language", fallback_guess_language)
+            gsl = guess_language(slm, content_type="text/x-patch")
             if have_gconf:
-                self.apply_gedit_colors(gsl)
-            self.apply_colordiff_colors(gsl)
+                self.apply_gedit_colors(self.buffer)
+            self.apply_colordiff_colors(self.buffer)
             self.buffer.set_language(gsl)
-            self.buffer.set_highlight(True)
+            self.buffer.set_highlight_syntax(True)
 
-            self.sourceview = gtksourceview.SourceView(self.buffer)
+            self.sourceview = gtksourceview2.View(self.buffer)
         else:
             self.buffer = gtk.TextBuffer()
             self.sourceview = gtk.TextView(self.buffer)
@@ -84,124 +94,94 @@ class DiffFileView(gtk.ScrolledWindow):
         self.sourceview.show()
 
     @staticmethod
-    def apply_gedit_colors(lang):
-        """Set style for lang to that specified in gedit configuration.
+    def apply_gedit_colors(buf):
+        """Set style to that specified in gedit configuration.
 
         This method needs the gconf module.
 
-        :param lang: a gtksourceview.SourceLanguage object.
+        :param buf: a gtksourceview2.Buffer object.
         """
-        GEDIT_SYNTAX_PATH = '/apps/gedit-2/preferences/syntax_highlighting'
-        GEDIT_LANG_PATH = GEDIT_SYNTAX_PATH + '/' + lang.get_id()
+        GEDIT_SCHEME_PATH = '/apps/gedit-2/preferences/editor/colors/scheme'
 
         client = gconf.client_get_default()
-        client.add_dir(GEDIT_LANG_PATH, gconf.CLIENT_PRELOAD_NONE)
-
-        for tag in lang.get_tags():
-            tag_id = tag.get_id()
-            gconf_key = GEDIT_LANG_PATH + '/' + tag_id
-            style_string = client.get_string(gconf_key)
-
-            if style_string is None:
-                continue
-
-            # function to get a bool from a string that's either '0' or '1'
-            string_bool = lambda x: bool(int(x))
-
-            # style_string is a string like "2/#FFCCAA/#000000/0/1/0/0"
-            # values are: mask, fg, bg, italic, bold, underline, strike
-            # this packs them into (str_value, attr_name, conv_func) tuples
-            items = zip(style_string.split('/'), ['mask', 'foreground',
-                'background', 'italic', 'bold', 'underline', 'strikethrough' ],
-                [ int, gtk.gdk.color_parse, gtk.gdk.color_parse, string_bool,
-                    string_bool, string_bool, string_bool ]
-            )
-
-            style = gtksourceview.SourceTagStyle()
-
-            # XXX The mask attribute controls whether the present values of
-            # foreground and background color should in fact be used. Ideally
-            # (and that's what gedit does), one could set all three attributes,
-            # and let the TagStyle object figure out which colors to use.
-            # However, in the GtkSourceview python bindings, the mask attribute
-            # is read-only, and it's derived instead from the colors being
-            # set or not. This means that we have to sometimes refrain from
-            # setting fg or bg colors, depending on the value of the mask.
-            # This code could go away if mask were writable.
-            mask = int(items[0][0])
-            if not (mask & 1): # GTK_SOURCE_TAG_STYLE_USE_BACKGROUND
-                items[2:3] = []
-            if not (mask & 2): # GTK_SOURCE_TAG_STYLE_USE_FOREGROUND
-                items[1:2] = []
-            items[0:1] = [] # skip the mask unconditionally
-
-            for value, attr, func in items:
-                try:
-                    value = func(value)
-                except ValueError:
-                    warning('gconf key %s contains an invalid value: %s'
-                            % gconf_key, value)
-                else:
-                    setattr(style, attr, value)
-
-            lang.set_tag_style(tag_id, style)
+        style_scheme_name = client.get_string(GEDIT_SCHEME_PATH)
+        if style_scheme_name is not None:
+            style_scheme = gtksourceview2.StyleSchemeManager().get_scheme(style_scheme_name)
+            
+            buf.set_style_scheme(style_scheme)
 
     @classmethod
-    def apply_colordiff_colors(klass, lang):
+    def apply_colordiff_colors(klass, buf):
         """Set style colors for lang using the colordiff configuration file.
 
         Both ~/.colordiffrc and ~/.colordiffrc.bzr-gtk are read.
 
-        :param lang: a "Diff" gtksourceview.SourceLanguage object.
+        :param buf: a "Diff" gtksourceview2.Buffer object.
         """
-        colors = {}
+        scheme_manager = gtksourceview2.StyleSchemeManager()
+        style_scheme = scheme_manager.get_scheme('colordiff')
+        
+        # if style scheme not found, we'll generate it from colordiffrc
+        # TODO: reload if colordiffrc has changed.
+        if style_scheme is None:
+            colors = {}
 
-        for f in ('~/.colordiffrc', '~/.colordiffrc.bzr-gtk'):
-            f = os.path.expanduser(f)
-            if os.path.exists(f):
-                try:
-                    f = file(f)
-                except IOError, e:
-                    warning('could not open file %s: %s' % (f, str(e)))
-                else:
-                    colors.update(klass.parse_colordiffrc(f))
-                    f.close()
+            for f in ('~/.colordiffrc', '~/.colordiffrc.bzr-gtk'):
+                f = os.path.expanduser(f)
+                if os.path.exists(f):
+                    try:
+                        f = file(f)
+                    except IOError, e:
+                        warning('could not open file %s: %s' % (f, str(e)))
+                    else:
+                        colors.update(klass.parse_colordiffrc(f))
+                        f.close()
 
-        if not colors:
-            # ~/.colordiffrc does not exist
-            return
-
-        mapping = {
-                # map GtkSourceView tags to colordiff names
+            if not colors:
+                # ~/.colordiffrc does not exist
+                return
+            
+            mapping = {
+                # map GtkSourceView2 scheme styles to colordiff names
                 # since GSV is richer, accept new names for extra bits,
                 # defaulting to old names if they're not present
-                'Added@32@line': ['newtext'],
-                'Removed@32@line': ['oldtext'],
-                'Location': ['location', 'diffstuff'],
-                'Diff@32@file': ['file', 'diffstuff'],
-                'Special@32@case': ['specialcase', 'diffstuff'],
-        }
-
-        for tag in lang.get_tags():
-            tag_id = tag.get_id()
-            keys = mapping.get(tag_id, [])
-            color = None
-
-            for key in keys:
-                color = colors.get(key, None)
-                if color is not None:
-                    break
-
-            if color is None:
-                continue
-
-            style = gtksourceview.SourceTagStyle()
-            try:
-                style.foreground = gtk.gdk.color_parse(color)
-            except ValueError:
-                warning('not a valid color: %s' % color)
-            else:
-                lang.set_tag_style(tag_id, style)
+                'diff:added-line': ['newtext'],
+                'diff:removed-line': ['oldtext'],
+                'diff:location': ['location', 'diffstuff'],
+                'diff:file': ['file', 'diffstuff'],
+                'diff:special-case': ['specialcase', 'diffstuff'],
+            }
+            
+            converted_colors = {}
+            for name, values in mapping.items():
+                color = None
+                for value in values:
+                    color = colors.get(value, None)
+                    if color is not None:
+                        break
+                if color is None:
+                    continue
+                converted_colors[name] = color
+            
+            # some xml magic to produce needed style scheme description
+            e_style_scheme = Element('style-scheme')
+            e_style_scheme.set('id', 'colordiff')
+            e_style_scheme.set('_name', 'ColorDiff')
+            e_style_scheme.set('version', '1.0')
+            for name, color in converted_colors.items():
+                style = SubElement(e_style_scheme, 'style')
+                style.set('name', name)
+                style.set('foreground', '#%s' % color)
+            
+            scheme_xml = tostring(e_style_scheme, 'UTF-8')
+            if not os.path.exists(os.path.expanduser('~/.local/share/gtksourceview-2.0/styles')):
+                os.makedirs(os.path.expanduser('~/.local/share/gtksourceview-2.0/styles'))
+            file(os.path.expanduser('~/.local/share/gtksourceview-2.0/styles/colordiff.xml'), 'w').write(scheme_xml)
+            
+            scheme_manager.force_rescan()
+            style_scheme = scheme_manager.get_scheme('colordiff')
+        
+        buf.set_style_scheme(style_scheme)
 
     @staticmethod
     def parse_colordiffrc(fileobj):
@@ -635,6 +615,7 @@ def iter_changes_to_status(source, target):
     renamed_and_modified = 'renamed and modified'
     modified = 'modified'
     kind_changed = 'kind changed'
+    missing = 'missing'
 
     # TODO: Handle metadata changes
 
@@ -655,9 +636,15 @@ def iter_changes_to_status(source, target):
                     source_marker = ''
                 else:
                     source_marker = osutils.kind_marker(kinds[0])
+
                 if kinds[1] is None:
-                    assert kinds[0] is not None
-                    marker = osutils.kind_marker(kinds[0])
+                    if kinds[0] is None:
+                        # We assume bzr will flag only files in that case,
+                        # there may be a bzr bug there as only files seems to
+                        # not receive any kind.
+                        marker = osutils.kind_marker('file')
+                    else:
+                        marker = osutils.kind_marker(kinds[0])
                 else:
                     marker = osutils.kind_marker(kinds[1])
 
@@ -665,17 +652,20 @@ def iter_changes_to_status(source, target):
                 if real_path is None:
                     real_path = paths[0]
                 assert real_path is not None
-                display_path = real_path + marker
 
                 present_source = versioned[0] and kinds[0] is not None
                 present_target = versioned[1] and kinds[1] is not None
 
-                if present_source != present_target:
+                if kinds[0] is None and kinds[1] is None:
+                    change_type = missing
+                    display_path = real_path + marker
+                elif present_source != present_target:
                     if present_target:
                         change_type = added
                     else:
                         assert present_source
                         change_type = removed
+                    display_path = real_path + marker
                 elif names[0] != names[1] or parent_ids[0] != parent_ids[1]:
                     # Renamed
                     if changed_content or executables[0] != executables[1]:
@@ -689,8 +679,9 @@ def iter_changes_to_status(source, target):
                     change_type = kind_changed
                     display_path = (paths[0] + source_marker
                                     + ' => ' + paths[1] + marker)
-                elif changed_content is True or executables[0] != executables[1]:
+                elif changed_content or executables[0] != executables[1]:
                     change_type = modified
+                    display_path = real_path + marker
                 else:
                     assert False, "How did we get here?"
 
