@@ -30,13 +30,13 @@ import gtk
 import bzrlib.errors as errors
 from bzrlib.workingtree import WorkingTree
 
+from bzrlib.plugins.gtk import _i18n
 from bzrlib.plugins.gtk.dialog import error_dialog, info_dialog, question_dialog, warning_dialog
 from bzrlib.plugins.gtk.errors import show_bzr_error
 from bzrlib.plugins.gtk.annotate.gannotate import GAnnotateWindow
 from bzrlib.plugins.gtk.annotate.config import GAnnotateConfig
 from bzrlib.plugins.gtk.diff import DiffWindow
-from launch import launch
-from bzrlib.plugins.gtk.olive import Preferences
+from bzrlib.plugins.gtk.olive import Preferences, launch
 
 class OliveMenu:
     """ This class is responsible for building the context menus. """
@@ -59,76 +59,76 @@ class OliveMenu:
         
         self.actiongroup = gtk.ActionGroup('context')
         self.actiongroup.add_actions([('add', gtk.STOCK_ADD,
-                                       _('Add'), None,
-                                       _('Add the selected file'),
+                                       _i18n('Add'), None,
+                                       _i18n('Add the selected file'),
                                        self.add_file),
                                       ('remove', gtk.STOCK_REMOVE,
-                                       _('Remove'), None,
-                                       _('Remove the selected file'),
+                                       _i18n('Remove'), None,
+                                       _i18n('Remove the selected file'),
                                        self.remove_file),
                                       ('remove_and_delete', gtk.STOCK_REMOVE,
-                                       _('Remove and delete'), None,
-                                       _('Remove the selected file/dir and delete from disk'),
+                                       _i18n('Remove and delete'), None,
+                                       _i18n('Remove the selected file/dir and delete from disk'),
                                        self.remove_and_delete_file),
                                       ('rename', None,
-                                       _('Rename'), None,
-                                       _('Rename the selected file'),
+                                       _i18n('Rename'), None,
+                                       _i18n('Rename the selected file'),
                                        self.rename_file),
                                       ('open', gtk.STOCK_OPEN,
-                                       _('Open'), None,
-                                       _('Open the selected file'),
+                                       _i18n('Open'), None,
+                                       _i18n('Open the selected file'),
                                        self.open_file),
                                       ('revert', None,
-                                       _('Revert'), None,
-                                       _('Revert the changes'),
+                                       _i18n('Revert'), None,
+                                       _i18n('Revert the changes'),
                                        self.revert),
                                       ('commit', None,
-                                       _('Commit'), None,
-                                       _('Commit the changes'),
+                                       _i18n('Commit'), None,
+                                       _i18n('Commit the changes'),
                                        self.commit),
                                       ('annotate', None,
-                                       _('Annotate'), None,
-                                       _('Annotate the selected file'),
+                                       _i18n('Annotate'), None,
+                                       _i18n('Annotate the selected file'),
                                        self.annotate),
                                       ('diff', None,
-                                       _('Diff'), None,
-                                       _('Show the diff of the file'),
+                                       _i18n('Diff'), None,
+                                       _i18n('Show the diff of the file'),
                                        self.diff),
                                       ('bookmark', None,
-                                       _('Bookmark'), None,
-                                       _('Bookmark current location'),
+                                       _i18n('Bookmark'), None,
+                                       _i18n('Bookmark current location'),
                                        self.bookmark),
                                       ('edit_bookmark', gtk.STOCK_EDIT,
-                                       _('Edit'), None,
-                                       _('Edit the selected bookmark'),
+                                       _i18n('Edit'), None,
+                                       _i18n('Edit the selected bookmark'),
                                        self.edit_bookmark),
                                       ('remove_bookmark', gtk.STOCK_REMOVE,
-                                       _('Remove'), None,
-                                       _('Remove the selected bookmark'),
+                                       _i18n('Remove'), None,
+                                       _i18n('Remove the selected bookmark'),
                                        self.remove_bookmark),
                                       ('open_folder', gtk.STOCK_OPEN,
-                                       _('Open Folder'), None,
-                                       _('Open bookmark folder in Nautilus'),
+                                       _i18n('Open Folder'), None,
+                                       _i18n('Open bookmark folder in Nautilus'),
                                        self.open_folder),
                                       ('diff_selected', None,
-                                       _('Selected...'), None,
-                                       _('Show the differences of the selected file'),
+                                       _i18n('Selected...'), None,
+                                       _i18n('Show the differences of the selected file'),
                                        self.diff_selected),
                                       ('diff_all', None,
-                                       _('All...'), None,
-                                       _('Show the differences of all files'),
+                                       _i18n('All...'), None,
+                                       _i18n('Show the differences of all files'),
                                        self.diff_all),
                                       ('view_remote', None,
-                                       _('View contents'), None,
-                                       _('View the contents of the file in a builtin viewer'),
+                                       _i18n('View contents'), None,
+                                       _i18n('View the contents of the file in a builtin viewer'),
                                        self.view_remote),
                                       ('diff_remote', None,
-                                       _('Show differences'), None,
-                                       _('Show the differences between two revisions of the file'),
+                                       _i18n('Show differences'), None,
+                                       _i18n('Show the differences between two revisions of the file'),
                                        self.diff_remote),
                                       ('revert_remote', None,
-                                       _('Revert to this revision'), None,
-                                       _('Revert the selected file to the selected revision'),
+                                       _i18n('Revert to this revision'), None,
+                                       _i18n('Revert the selected file to the selected revision'),
                                        self.revert_remote)
                                      ])
         
@@ -174,11 +174,14 @@ class OliveMenu:
         filename = self.selected
             
         if filename is None:
-            error_dialog(_('No file was selected'),
-                         _('Please select a file from the list,\nor choose the other option.'))
+            error_dialog(_i18n('No file was selected'),
+                         _i18n('Please select a file from the list,\nor choose the other option.'))
             return
         
-        bzrlib.add.smart_add([os.path.join(directory, filename)])
+        wt, path = WorkingTree.open_containing(os.path.join(directory, filename))
+        wt.add([path])
+        
+        self.app.refresh_right()
     
     @show_bzr_error
     def annotate(self, action):
@@ -187,15 +190,16 @@ class OliveMenu:
         filename = self.selected
         
         if filename is None:
-            error_dialog(_('No file was selected'),
-                         _('Please select a file from the list.'))
+            error_dialog(_i18n('No file was selected'),
+                         _i18n('Please select a file from the list.'))
             return
         
         wt, path = WorkingTree.open_containing(os.path.join(directory, filename))
         
         branch = wt.branch
         file_id = wt.path2id(wt.relpath(os.path.join(directory, filename)))
-        
+        if file_id is None:
+            raise errors.NotVersionedError(filename)
         window = GAnnotateWindow(all=False, plain=False, parent=self.app)
         window.set_title(os.path.join(directory, filename) + " - Annotate")
         config = GAnnotateConfig(window)
@@ -207,15 +211,15 @@ class OliveMenu:
             branch.unlock()
     
     @show_bzr_error
-    def remove_file(self, action,delete_on_disk=0):
+    def remove_file(self, action, delete_on_disk=False):
         """ Right context menu -> Remove """
         # Remove only the selected file
         directory = self.path
         filename = self.selected
         
         if filename is None:
-            error_dialog(_('No file was selected'),
-                         _('Please select a file from the list,\nor choose the other option.'))
+            error_dialog(_i18n('No file was selected'),
+                         _i18n('Please select a file from the list,\nor choose the other option.'))
             return
         
         wt, path = WorkingTree.open_containing(os.path.join(directory, filename))
@@ -224,7 +228,7 @@ class OliveMenu:
         if delete_on_disk:
             abs_filename = os.path.join(directory,filename)
             if os.path.isdir(abs_filename):
-                response = question_dialog(_('Delete directory with all directories below ?'), abs_filename )
+                response = question_dialog(_i18n('Delete directory with all directories below ?'), abs_filename )
                 if response == gtk.RESPONSE_YES:
                     shutil.rmtree(abs_filename)
             else:
@@ -235,14 +239,17 @@ class OliveMenu:
         
     def remove_and_delete_file(self, action):
         """ Right context menu -> Remove and delete"""
-        self.remove_file(action,delete_on_disk=1)
+        self.remove_file(action, delete_on_disk=True)
 
     def rename_file(self, action):
         """ Right context menu -> Rename """
-        from rename import OliveRename
-        wt = WorkingTree.open_containing(self.path + os.sep + self.selected)[0]
-        rename = OliveRename(wt, wt.relpath(self.path), self.selected)
-        rename.display()
+        from bzrlib.plugins.gtk.olive.rename import RenameDialog
+        wt = WorkingTree.open_containing(os.path.join(self.path, self.selected))[0]
+        rename = RenameDialog(wt, wt.relpath(self.path), self.selected)
+        response = rename.run()
+        rename.destroy()
+        if response == gtk.RESPONSE_OK:
+            self.app.refresh_right()
     
     def open_file(self, action):
         """ Right context menu -> Open """
@@ -250,32 +257,27 @@ class OliveMenu:
         filename = self.selected
         
         if filename is None:
-            error_dialog(_('No file was selected'),
-                         _('Please select a file from the list,\nor choose the other option.'))
+            error_dialog(_i18n('No file was selected'),
+                         _i18n('Please select a file from the list,\nor choose the other option.'))
             return
 
         if filename == '..':
             # TODO: how to enter a directory?
             return
         else:
-            fullpath = self.path + os.sep + filename
-            if os.path.isdir(fullpath):
-                # selected item is an existant directory
-                # TODO: how to enter a directory?
-                return
-            else:
-                launch(fullpath) 
+            fullpath = os.path.join(self.path, filename)
+            launch.launch(fullpath) 
 
     def revert(self, action):
         """ Right context menu -> Revert """
         wt, path = WorkingTree.open_containing(self.path)
         ret = wt.revert([os.path.join(path, self.selected)])
         if ret:
-            warning_dialog(_('Conflicts detected'),
-                           _('Please have a look at the working tree before continuing.'))
+            warning_dialog(_i18n('Conflicts detected'),
+                           _i18n('Please have a look at the working tree before continuing.'))
         else:
-            info_dialog(_('Revert successful'),
-                        _('All files reverted to last revision.'))
+            info_dialog(_i18n('Revert successful'),
+                        _i18n('Selected file reverted to last revision.'))
         self.app.refresh_right()       
     
     def commit(self, action):
@@ -308,19 +310,19 @@ class OliveMenu:
         wt = WorkingTree.open_containing(self.path)[0]
         window = DiffWindow(self.app)
         parent_tree = wt.branch.repository.revision_tree(wt.branch.last_revision())
-        window.set_diff(wt.branch.nick, wt, parent_tree)
+        window.set_diff(wt.branch._get_nick(local=True), wt, parent_tree)
         window.set_file(wt.relpath(self.path + os.sep + self.selected))
         window.show()
     
     def bookmark(self, action):
         """ Right context menu -> Bookmark """
         if self.pref.add_bookmark(self.path):
-            info_dialog(_('Bookmark successfully added'),
-                        _('The current directory was bookmarked. You can reach\nit by selecting it from the left panel.'))
+            info_dialog(_i18n('Bookmark successfully added'),
+                        _i18n('The current directory was bookmarked. You can reach\nit by selecting it from the left panel.'))
             self.pref.write()
         else:
-            warning_dialog(_('Location already bookmarked'),
-                           _('The current directory is already bookmarked.\nSee the left panel for reference.'))
+            warning_dialog(_i18n('Location already bookmarked'),
+                           _i18n('The current directory is already bookmarked.\nSee the left panel for reference.'))
         
         self.app.refresh_left()
 
@@ -354,7 +356,7 @@ class OliveMenu:
         path = self.selected
 
         if path != None:
-            launch(path)
+            launch.launch(path)
     
     def diff_selected(self, action):
         """ Diff toolbutton -> Selected... """
@@ -362,9 +364,11 @@ class OliveMenu:
     
     def diff_all(self, action):
         """ Diff toolbutton -> All... """
-        from diff import OliveDiff
-        diff = OliveDiff(self.comm)
-        diff.display()
+        wt = WorkingTree.open_containing(self.path)[0]
+        window = DiffWindow(self.app)
+        parent_tree = wt.branch.repository.revision_tree(wt.branch.last_revision())
+        window.set_diff(wt.branch._get_nick(local=True), wt, parent_tree)
+        window.show()
     
     def view_remote(self, action):
         """ Remote context menu -> View contents """
