@@ -1,4 +1,4 @@
-# Copyright (C) 2007 John Arbash Meinel <john@arbash-meinel.com>
+# Copyright (C) 2007, 2008 John Arbash Meinel <john@arbash-meinel.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,10 +21,15 @@ import os
 import gtk
 
 from bzrlib import (
-    tests,
+    branch,
     revision,
+    tests,
+    uncommit,
     )
-from bzrlib.util import bencode
+try:
+    from bzrlib import bencode
+except ImportError:
+    from bzrlib.util import bencode
 
 from bzrlib.plugins.gtk import commit
 
@@ -644,14 +649,35 @@ class TestCommitDialog(tests.TestCaseWithTransport):
                             'message':'message\nfor b_dir\n'},
                           ]), dlg._get_specific_files())
 
+    def test_specific_files_sanitizes_messages(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.branch.get_config().set_user_option('per_file_commits', 'true')
+        self.build_tree(['tree/a_file', 'tree/b_dir/'])
+        tree.add(['a_file', 'b_dir'], ['1a-id', '0b-id'])
 
-class TestCommitDialog_Commit(tests.TestCaseWithTransport):
-    """Tests on the actual 'commit' button being pushed."""
+        dlg = commit.CommitDialog(tree)
+        dlg._commit_selected_radio.set_active(True)
+        self.assertEqual((['a_file', 'b_dir'], []), dlg._get_specific_files())
+
+        dlg._treeview_files.set_cursor((1,))
+        dlg._set_file_commit_message('Test\r\nmessage\rfor a_file\n')
+        dlg._treeview_files.set_cursor((2,))
+        dlg._set_file_commit_message('message\r\nfor\nb_dir\r')
+
+        self.assertEqual((['a_file', 'b_dir'],
+                          [{'path':'a_file', 'file_id':'1a-id',
+                            'message':'Test\nmessage\nfor a_file\n'},
+                           {'path':'b_dir', 'file_id':'0b-id',
+                            'message':'message\nfor\nb_dir\n'},
+                          ]), dlg._get_specific_files())
+
+
+class QuestionHelpers(object):
 
     def _set_question_yes(self, dlg):
         """Set the dialog to answer YES to any questions."""
         self.questions = []
-        def _question_yes(*args):
+        def _question_yes(*args, **kwargs):
             self.questions.append(args)
             self.questions.append('YES')
             return gtk.RESPONSE_YES
@@ -660,12 +686,16 @@ class TestCommitDialog_Commit(tests.TestCaseWithTransport):
     def _set_question_no(self, dlg):
         """Set the dialog to answer NO to any questions."""
         self.questions = []
-        def _question_no(*args):
+        def _question_no(*args, **kwargs):
             self.questions.append(args)
             self.questions.append('NO')
             return gtk.RESPONSE_NO
         dlg._question_dialog = _question_no
 
+
+class TestCommitDialog_Commit(tests.TestCaseWithTransport, QuestionHelpers):
+    """Tests on the actual 'commit' button being pushed."""
+
     def test_bound_commit_local(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/a'])
@@ -687,6 +717,21 @@ class TestCommitDialog_Commit(tests.TestCaseWithTransport):
         self.assertEqual(last_rev, dlg.committed_revision_id)
         self.assertEqual(rev_id1, tree.branch.last_revision())
 
+    def test_commit_global_sanitizes_message(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+        rev_id1 = tree.commit('one')
+
+        self.build_tree(['tree/b'])
+        tree.add(['b'], ['b-id'])
+        dlg = commit.CommitDialog(tree)
+        # With the check box set, it should only effect the local branch
+        dlg._set_global_commit_message('Commit\r\nmessage\rfoo\n')
+        dlg._do_commit()
+        rev = tree.branch.repository.get_revision(tree.last_revision())
+        self.assertEqual('Commit\nmessage\nfoo\n', rev.message)
+
     def test_bound_commit_both(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/a'])
@@ -708,7 +753,7 @@ class TestCommitDialog_Commit(tests.TestCaseWithTransport):
         self.assertEqual(last_rev, dlg.committed_revision_id)
         self.assertEqual(last_rev, tree.branch.last_revision())
 
-    def test_commit_no_message(self):
+    def test_commit_empty_message(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/a', 'tree/b'])
         tree.add(['a'], ['a-id'])
@@ -943,13 +988,14 @@ class TestCommitDialog_Commit(tests.TestCaseWithTransport):
         rev = tree.branch.repository.get_revision(rev_id2)
         self.assertEqual('Commit just "a"', rev.message)
         file_info = rev.properties['file-info']
-        self.assertEqual('ld7:file_id4:a-id'
-                           '7:message14:Message for A\n'
-                           '4:path1:a'
-                         'ee', file_info)
+        self.assertEqual(u'ld7:file_id4:a-id'
+                         '7:message14:Message for A\n'
+                         '4:path1:a'
+                         'ee',
+                         file_info)
         self.assertEqual([{'path':'a', 'file_id':'a-id',
-                           'message':'Message for A\n'},
-                         ], bencode.bdecode(file_info))
+                           'message':'Message for A\n'},],
+                         bencode.bdecode(file_info.encode('UTF-8')))
 
     def test_commit_messages_after_merge(self):
         tree = self.make_branch_and_tree('tree')
@@ -976,13 +1022,14 @@ class TestCommitDialog_Commit(tests.TestCaseWithTransport):
         self.assertEqual('Merging from "tree2"\n', rev.message)
         self.assertEqual([rev_id1, rev_id2], rev.parent_ids)
         file_info = rev.properties['file-info']
-        self.assertEqual('ld7:file_id4:a-id'
-                           '7:message14:Message for A\n'
-                           '4:path1:a'
-                         'ee', file_info)
+        self.assertEqual(u'ld7:file_id4:a-id'
+                         '7:message14:Message for A\n'
+                         '4:path1:a'
+                         'ee',
+                         file_info)
         self.assertEqual([{'path':'a', 'file_id':'a-id',
-                           'message':'Message for A\n'},
-                         ], bencode.bdecode(file_info))
+                           'message':'Message for A\n'},],
+                         bencode.bdecode(file_info.encode('UTF-8')))
 
     def test_commit_unicode_messages(self):
         self.requireFeature(tests.UnicodeFilenameFeature)
@@ -1030,3 +1077,153 @@ class TestCommitDialog_Commit(tests.TestCaseWithTransport):
                           {'path':u'\u03a9', 'file_id':'omega-id',
                            'message':u'\u03a9 is the end of all things.\n'},
                          ], file_info_decoded)
+
+
+class TestSanitizeMessage(tests.TestCase):
+
+    def assertSanitize(self, expected, original):
+        self.assertEqual(expected,
+                         commit._sanitize_and_decode_message(original))
+
+    def test_untouched(self):
+        self.assertSanitize('foo\nbar\nbaz\n', 'foo\nbar\nbaz\n')
+
+    def test_converts_cr_to_lf(self):
+        self.assertSanitize('foo\nbar\nbaz\n', 'foo\rbar\rbaz\r')
+
+    def test_converts_crlf_to_lf(self):
+        self.assertSanitize('foo\nbar\nbaz\n', 'foo\r\nbar\r\nbaz\r\n')
+
+    def test_converts_mixed_to_lf(self):
+        self.assertSanitize('foo\nbar\nbaz\n', 'foo\r\nbar\rbaz\n')
+
+
+class TestSavedCommitMessages(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestSavedCommitMessages, self).setUp()
+        # Install our hook
+        branch.Branch.hooks.install_named_hook(
+            'post_uncommit', commit.save_commit_messages, None)
+
+    def _get_file_info_dict(self, rank):
+        file_info = [dict(path='a', file_id='a-id', message='a msg %d' % rank),
+                     dict(path='b', file_id='b-id', message='b msg %d' % rank)]
+        return file_info
+
+    def _get_file_info_revprops(self, rank):
+        file_info_prop = self._get_file_info_dict(rank)
+        return {'file-info': bencode.bencode(file_info_prop).decode('UTF-8')}
+
+    def _get_commit_message(self):
+        return self.config.get_user_option('gtk_global_commit_message')
+
+    def _get_file_commit_messages(self):
+        return self.config.get_user_option('gtk_file_commit_messages')
+
+
+class TestUncommitHook(TestSavedCommitMessages):
+
+    def setUp(self):
+        super(TestUncommitHook, self).setUp()
+        self.tree = self.make_branch_and_tree('tree')
+        self.config = self.tree.branch.get_config()
+        self.build_tree(['tree/a', 'tree/b'])
+        self.tree.add(['a'], ['a-id'])
+        self.tree.add(['b'], ['b-id'])
+        rev1 = self.tree.commit('one', rev_id='one-id',
+                                revprops=self._get_file_info_revprops(1))
+        rev2 = self.tree.commit('two', rev_id='two-id',
+                                revprops=self._get_file_info_revprops(2))
+        rev3 = self.tree.commit('three', rev_id='three-id',
+                                revprops=self._get_file_info_revprops(3))
+
+    def test_uncommit_one_by_one(self):
+        uncommit.uncommit(self.tree.branch, tree=self.tree)
+        self.assertEquals(u'three', self._get_commit_message())
+        self.assertEquals(u'd4:a-id7:a msg 34:b-id7:b msg 3e',
+                          self._get_file_commit_messages())
+
+        uncommit.uncommit(self.tree.branch, tree=self.tree)
+        self.assertEquals(u'two\n******\nthree', self._get_commit_message())
+        self.assertEquals(u'd4:a-id22:a msg 2\n******\na msg 3'
+                          '4:b-id22:b msg 2\n******\nb msg 3e',
+                          self._get_file_commit_messages())
+
+        uncommit.uncommit(self.tree.branch, tree=self.tree)
+        self.assertEquals(u'one\n******\ntwo\n******\nthree',
+                          self._get_commit_message())
+        self.assertEquals(u'd4:a-id37:a msg 1\n******\na msg 2\n******\na msg 3'
+                          '4:b-id37:b msg 1\n******\nb msg 2\n******\nb msg 3e',
+                          self._get_file_commit_messages())
+
+    def test_uncommit_all_at_once(self):
+        uncommit.uncommit(self.tree.branch, tree=self.tree, revno=1)
+        self.assertEquals(u'one\n******\ntwo\n******\nthree',
+                          self._get_commit_message())
+        self.assertEquals(u'd4:a-id37:a msg 1\n******\na msg 2\n******\na msg 3'
+                          '4:b-id37:b msg 1\n******\nb msg 2\n******\nb msg 3e',
+                          self._get_file_commit_messages())
+
+
+class TestReusingSavedCommitMessages(TestSavedCommitMessages, QuestionHelpers):
+
+    def setUp(self):
+        super(TestReusingSavedCommitMessages, self).setUp()
+        self.tree = self.make_branch_and_tree('tree')
+        self.config = self.tree.branch.get_config()
+        self.config.set_user_option('per_file_commits', 'true')
+        self.build_tree(['tree/a', 'tree/b'])
+        self.tree.add(['a'], ['a-id'])
+        self.tree.add(['b'], ['b-id'])
+        rev1 = self.tree.commit('one', revprops=self._get_file_info_revprops(1))
+        rev2 = self.tree.commit('two', revprops=self._get_file_info_revprops(2))
+        uncommit.uncommit(self.tree.branch, tree=self.tree)
+        self.build_tree_contents([('tree/a', 'new a content\n'),
+                                  ('tree/b', 'new b content'),])
+
+    def _get_commit_dialog(self, tree):
+        # Ensure we will never use a dialog that can actually prompt the user
+        # during the test suite. Test *can* and *should* override with the
+        # correct question dialog type.
+        dlg = commit.CommitDialog(tree)
+        self._set_question_no(dlg)
+        return dlg
+
+    def test_setup_saved_messages(self):
+        # Check the initial setup
+        self.assertEquals(u'two', self._get_commit_message())
+        self.assertEquals(u'd4:a-id7:a msg 24:b-id7:b msg 2e',
+                          self._get_file_commit_messages())
+
+    def test_messages_are_reloaded(self):
+        dlg = self._get_commit_dialog(self.tree)
+        self.assertEquals(u'two', dlg._get_global_commit_message())
+        self.assertEquals(([u'a', u'b'],
+                           [{ 'path': 'a',
+                             'file_id': 'a-id', 'message': 'a msg 2',},
+                           {'path': 'b',
+                            'file_id': 'b-id', 'message': 'b msg 2',}],),
+                          dlg._get_specific_files())
+
+    def test_messages_are_consumed(self):
+        dlg = self._get_commit_dialog(self.tree)
+        dlg._do_commit()
+        self.assertEquals(u'', self._get_commit_message())
+        self.assertEquals(u'de', self._get_file_commit_messages())
+
+    def test_messages_are_saved_on_cancel_if_required(self):
+        dlg = self._get_commit_dialog(self.tree)
+        self._set_question_yes(dlg) # Save messages
+        dlg._do_cancel()
+        self.assertEquals(u'two', self._get_commit_message())
+        self.assertEquals(u'd4:a-id7:a msg 24:b-id7:b msg 2e',
+                          self._get_file_commit_messages())
+
+    def test_messages_are_cleared_on_cancel_if_required(self):
+        dlg = self._get_commit_dialog(self.tree)
+        self._set_question_no(dlg) # Don't save messages
+        dlg._do_cancel()
+        self.assertEquals(u'', self._get_commit_message())
+        self.assertEquals(u'de',
+                          self._get_file_commit_messages())
