@@ -48,14 +48,12 @@ class BranchWindow(Window):
         self.maxnum      = maxnum
         self.config      = GlobalConfig()
 
-        self._sizes      = {} # window and widget sizes
-
         if self.config.get_user_option('viz-compact-view') == 'yes':
             self.compact_view = True
         else:
             self.compact_view = False
 
-        self.set_title(branch.nick + " - revision history")
+        self.set_title(branch._get_nick(local=True) + " - revision history")
 
         # user-configured window size
         size = self._load_size('viz-window-size')
@@ -69,7 +67,7 @@ class BranchWindow(Window):
             height = int(monitor.height * 0.75)
         self.set_default_size(width, height)
         self.set_size_request(width/3, height/3)
-        self.connect("size-allocate", self._on_size_allocate, 'viz-window-size')
+        self._save_size_on_destroy(self, 'viz-window-size')
 
         # FIXME AndyFitz!
         icon = self.render_icon(gtk.STOCK_INDEX, gtk.ICON_SIZE_BUTTON)
@@ -104,6 +102,15 @@ class BranchWindow(Window):
 
         self.construct()
 
+    def _save_size_on_destroy(self, widget, config_name):
+        """Creates a hook that saves the size of widget to config option 
+           config_name when the window is destroyed/closed."""
+        def save_size(src):
+            width, height = widget.allocation.width, widget.allocation.height
+            value = '%sx%s' % (width, height)
+            self.config.set_user_option(config_name, value)
+        self.connect("destroy", save_size)
+
     def set_revision(self, revid):
         self.treeview.set_revision_id(revid)
 
@@ -125,6 +132,12 @@ class BranchWindow(Window):
         vbox.pack_start(self.paned, expand=True, fill=True)
         vbox.set_focus_child(self.paned)
 
+        self.treeview.connect('revision-selected',
+                self._treeselection_changed_cb)
+        self.treeview.connect('revision-activated',
+                self._tree_revision_activated)
+
+        self.treeview.connect('tag-added', lambda w, t, r: self._update_tags())
         vbox.show()
     
     def construct_menubar(self):
@@ -289,13 +302,6 @@ class BranchWindow(Window):
 
         self.treeview = TreeView(self.branch, self.start_revs, self.maxnum, self.compact_view)
 
-        self.treeview.connect('revision-selected',
-                self._treeselection_changed_cb)
-        self.treeview.connect('revision-activated',
-                self._tree_revision_activated)
-
-        self.treeview.connect('tag-added', lambda w, t, r: self._update_tags())
-
         for col in ["revno", "date"]:
             option = self.config.get_user_option(col + '-column-visible')
             if option is not None:
@@ -316,7 +322,7 @@ class BranchWindow(Window):
         else:
             (width, height) = self.get_size()
             align.set_size_request(width, int(height / 2.5))
-        align.connect('size-allocate', self._on_size_allocate, 'viz-graph-size')
+        self._save_size_on_destroy(align, 'viz-graph-size')
         align.show()
 
         return align
@@ -359,7 +365,7 @@ class BranchWindow(Window):
         if size:
             width, height = size
             self.revisionview.set_size_request(width, height)
-        self.revisionview.connect('size-allocate', self._on_size_allocate, 'viz-revisionview-size')
+        self._save_size_on_destroy(self.revisionview, 'viz-revisionview-size')
         self.revisionview.show()
         self.revisionview.set_show_callback(self._show_clicked_cb)
         self.revisionview.connect('notify::revision', self._go_clicked_cb)
@@ -440,7 +446,7 @@ class BranchWindow(Window):
         parents  = self.treeview.get_parents()
 
         if len(parents) == 0:
-            parent_id = None
+            parent_id = NULL_REVISION
         else:
             parent_id = parents[0]
 
@@ -598,37 +604,18 @@ class BranchWindow(Window):
         if size:
             width, height = [int(num) for num in size.split('x')]
             # avoid writing config every time we start
-            self._sizes[name] = (width, height)
             return width, height
         return None
 
-    def _on_size_allocate(self, widget, allocation, name):
-        """When window has been resized, save the new size."""
-        width, height = 0, 0
-        if name in self._sizes:
-            width, height = self._sizes[name]
-
-        size_changed = (width != allocation.width) or \
-                (height != allocation.height)
-
-        if size_changed:
-            width, height = allocation.width, allocation.height
-            self._sizes[name] = (width, height)
-            value = '%sx%s' % (width, height)
-            self.config.set_user_option(name, value)
-
-    def show_diff(self, revid=None, parentid=None):
+    def show_diff(self, revid=None, parentid=NULL_REVISION):
         """Open a new window to show a diff between the given revisions."""
         from bzrlib.plugins.gtk.diff import DiffWindow
         window = DiffWindow(parent=self)
 
-        if parentid is None:
-            parentid = NULL_REVISION
-
         rev_tree    = self.branch.repository.revision_tree(revid)
         parent_tree = self.branch.repository.revision_tree(parentid)
 
-        description = revid + " - " + self.branch.nick
+        description = revid + " - " + self.branch._get_nick(local=True)
         window.set_diff(description, rev_tree, parent_tree)
         window.show()
 
@@ -639,13 +626,13 @@ class BranchWindow(Window):
 
         if not revision: # default to selected row
             revision = self.treeview.get_revision()
-        if (not revision) or (revision == NULL_REVISION):
+        if revision == NULL_REVISION:
             return
 
         if not parents: # default to selected row's parents
             parents  = self.treeview.get_parents()
         if len(parents) == 0:
-            parent_id = None
+            parent_id = NULL_REVISION
         else:
             parent_id = parents[0]
 
